/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 2012 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
 * or packager/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at packager/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package org.glassfish.osgicdi.impl;

import org.jboss.weld.bootstrap.api.Bootstrap;
import org.jboss.weld.bootstrap.spi.BeanDeploymentArchive;
import org.osgi.framework.Bundle;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.logging.Logger;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

/**
 * Factory for {@link OSGiBeanDeploymentArchiveImpl} used by {@link OSGiCDIDeploymentRequest}.
 * <p/>
 * It scans bundle in order to verify it is a CDI manageable bundle and to
 * create a complete {@link OSGiBeanDeploymentArchiveImpl}.
 *
 * @author Tang Yong(tangyong@cn.fujitsu.com)
 */
public class OSGiBeanDeploymentArchiveFactory {
	private static Logger logger = Logger.getLogger(OSGiBeanDeploymentArchiveFactory.class.getPackage().getName());

    private Set<String> beanClasses = new HashSet<String>();

    private List<URL> beanXmlUrls = new ArrayList<URL>();

    private Bundle bundle = null;

    public BeanDeploymentArchive scan(Bundle bundle, Bootstrap bootstrap) {        
        this.bundle = bundle;
        beanXmlUrls.clear();
        beanClasses.clear();

        Enumeration<?> beansXmlMarkers = bundle.findEntries("/", "beans.xml", true);
        Enumeration<?> innerJars = bundle.findEntries("/", "*.jar", true);
        Enumeration<?> innerZips = bundle.findEntries("/", "*.zip", true);

        if (beansXmlMarkers != null) {
            scanRoot(beansXmlMarkers);
        }
        if (innerZips != null) {
            while (innerZips.hasMoreElements()) {
                scanZip((URL) innerZips.nextElement());
            }
        }
        if (innerJars != null) {
            while (innerJars.hasMoreElements()) {
                scanZip((URL) innerJars.nextElement());
            }
        }

        if (beanXmlUrls.size() < 1) {
            return null;
        }

        OSGiBeanDeploymentArchiveImpl archive =
                new OSGiBeanDeploymentArchiveImpl("osgi-bean-deployment-archive-"
                        + bundle.getBundleId());
        archive.setBeansXml(bootstrap.parse(beanXmlUrls));
        archive.setBeanClasses(beanClasses);

        return archive;
    }

    private void scanZip(URL zipUrl) {
        ZipInputStream zipInputStream = null;
        ZipEntry zipEntry = null;
        String zipEntryPath = "";
        Set<String> zipClasses = new HashSet<String>();
        List<URL> zipBeanXmlUrls = new ArrayList<URL>();
        try {
            zipInputStream = new ZipInputStream(zipUrl.openStream());
            zipEntry = zipInputStream.getNextEntry();
            while (zipEntry != null) {
                zipEntryPath = "/" + zipEntry.getName();
                if (zipEntryPath.toLowerCase().endsWith(".zip")
                        || zipEntryPath.toLowerCase().endsWith(".jar")) {
                    scanZip(new URL("jar:" + zipUrl + "!" + zipEntryPath));
                } else if (zipEntryPath.toLowerCase().endsWith(".class")) {
                    String clazz = null;
                    String[] parts = zipEntryPath.split("!");
                    if (parts.length > 1) {
                        clazz = parts[1].substring(1).
                            replace("/", ".").
                            replace(".class", "");
                    } else {
                        clazz = zipEntryPath.substring(1).
                            replace("/", ".").
                            replace(".class", "");
                    }
                    zipClasses.add(clazz);
                } else if (zipEntryPath.toLowerCase().endsWith("beans.xml")) {
                    zipBeanXmlUrls.add(new URL("jar:" + zipUrl + "!"
                            + zipEntryPath));
                }
                zipEntry = zipInputStream.getNextEntry();
            }
            if (zipBeanXmlUrls.size() > 1) {
            	beanXmlUrls.addAll(zipBeanXmlUrls);
                beanClasses.addAll(zipClasses);
            }
        } catch (MalformedURLException e) {
            e.printStackTrace();
        } catch (IOException e) {
        }
    }

    private void scanRoot(Enumeration<?> beansXmlMarkers) {
        while (beansXmlMarkers.hasMoreElements()) {
            URL url = (URL) beansXmlMarkers.nextElement();
            String beansXmlPath = url.getPath();
            if (!beansXmlPath.endsWith("META-INF/beans.xml")) {
                continue;
            }
            beanXmlUrls.add(url);
            Enumeration<?> classes = bundle.findEntries("/", "*.class", true);
            if (classes != null) {
                while (classes.hasMoreElements()) {
                    URL classUrl = (URL) classes.nextElement();
                    String clazz = null;
                    String[] parts = classUrl.getFile().split("!");
                    if (parts.length > 1) {
                        clazz = parts[1].substring(1).
                            replace("/", ".").
                            replace(".class", "");
                    } else {
                        clazz = classUrl.getFile().substring(1).
                            replace("/", ".").
                            replace(".class", "");
                    }
                    beanClasses.add(clazz);
                }
            }
        }
    }
}
